==> decibel_0.3.0.orig.tar.gz: decibel-0.3.0/src/server/connectionfacade.cpp <==
/*
 * the Decibel Realtime Communication Framework
 * Copyright (C) 2006 by basyskom GmbH
 *  @author Tobias Hunger <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <QtCore/QCoreApplication>
#include <QtCore/QPointer>

#include <QtTapioca/ConnectionManager>
#include <QtTapioca/ConnectionManagerFactory>
#include <QtTapioca/UserContact>

#include "connectionfacade.h"
#include "protocolmanager.h"

#include <Decibel/Errors>
#include <Decibel/AccountData>

/**
 * @brief Private data class for the ConnectionFacade.
 * @author Tobias Hunger <info@basyskom.de>
 */
class ConnectionFacadePrivate
{
public:
    /** @brief A pointer to the ProtocolManager. */
    QPointer<ProtocolManager> protocolMgr;

    /** @brief A list of known Connections. */
    QList<QtTapioca::Connection *> connections;
};

// -----------------------------------------------------------------------

ConnectionFacade::ConnectionFacade(ProtocolManager* protocol_mgr,
                                   QObject * parent) :
    QObject(parent),
    d(new ConnectionFacadePrivate())
{
    Q_ASSERT(protocol_mgr != 0);
    d->protocolMgr = protocol_mgr;
}

ConnectionFacade::~ConnectionFacade()
{
    // Disconnect from all Connections:
    QtTapioca::Connection * connection;
    foreach (connection, d->connections)
    { connection->disconnect(); }

    // Wait for disconnection events:
    // qDebug(qPrintable(tr("Waiting for all connections to close.")));
    // while (!d->connections.isEmpty())
    // { QCoreApplication::instance()->processEvents(); }
    // qDebug(qPrintable(tr("All connections closed: Exiting now.")));

    delete d;
}

QtTapioca::Connection *
ConnectionFacade::connectUsingAccountAndConnectionManager(const QVariantMap& account_data,
                                                          const QString cm_name,
                                                          const int presence_state,
                                                          const QString message)
{
    Q_ASSERT(!account_data.isEmpty());

    QString protocol(account_data[Decibel::name_protocol].toString());

    Q_ASSERT(presence_state != QtTapioca::ContactBase::Offline);
    // AccountManager should not have called us if the account is
    // supposed to be offline!
    Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
             presence_state <= QtTapioca::ContactBase::Busy);

    // Get CM:
    QtTapioca::ConnectionManager * current_cm =
            QtTapioca::ConnectionManagerFactory::self()->
            getConnectionManagerByName(cm_name);

    if (current_cm == 0) { return 0; }

    Q_ASSERT(current_cm != 0);

    // Map parameters:
    QList<QtTapioca::ConnectionManager::Parameter> cm_params =
            current_cm->protocolParameters(protocol);
    QList<QtTapioca::ConnectionManager::Parameter> account_params;
    QtTapioca::ConnectionManager::Parameter current_param;
    QString key;
    foreach (current_param, cm_params)
    {
        key = current_param.name();
        if (account_data.contains(key) &&
            account_data[key].typeName() ==
                current_param.value().typeName())
        {
            account_params.append(QtTapioca::ConnectionManager::Parameter(key,
                                                                          account_data[key]));
        }
        else if (current_param.flags() & QtTapioca::ConnectionManager::Parameter::Required)
        { return 0; }
    }

    // Create connection:
    QtTapioca::Connection* connection = current_cm->requestConnection(protocol, account_params);
    if (connection == 0) { return 0; }

    Q_ASSERT(connection != 0);

    d->connections.append(connection);

    // register connection:
    connect(connection,
            SIGNAL(statusChanged(QtTapioca::Connection*,
                                 QtTapioca::Connection::Status,
                                 QtTapioca::Connection::Reason)),
            SLOT(onStatusChanged(QtTapioca::Connection *,
                                 QtTapioca::Connection::Status,
                                 QtTapioca::Connection::Reason)));
    connect(connection,
            SIGNAL(channelCreated(QtTapioca::Connection*,
                                  QtTapioca::Channel*, bool)),
            SLOT(onChannelOpened(QtTapioca::Connection*,
                                 QtTapioca::Channel*, bool)));

    // do the actual connect:
    connection->connect(QtTapioca::UserContact::Presence(presence_state), message);

    return connection;
}

QtTapioca::Connection *
ConnectionFacade::connectUsingAccount(const QVariantMap& account_data,
                                      const int presence_state,
                                      const QString message)
{
    QString protocol = account_data[Decibel::name_protocol].toString();
    // Get a connection manager to use:
    QString cm_name(d->protocolMgr->defaultConnectionManagerForProtocol(protocol));
    if (cm_name.isEmpty())
    { return 0; }

    return connectUsingAccountAndConnectionManager(account_data, cm_name, presence_state, message);
}

void ConnectionFacade::onStatusChanged(QtTapioca::Connection* connection,
                                       QtTapioca::Connection::Status new_state,
                                       QtTapioca::Connection::Reason reason)
{
    // find connection_handle:
    if (connection == 0) { return; }

    // This is one of our connections:
    Q_ASSERT(connection != 0); // 0 connection may not end up in d->connections!

    if (new_state == QtTapioca::Connection::Connected)
    {
        connect(connection->userContact(),
                SIGNAL(presenceUpdated(QtTapioca::ContactBase*,
                                       QtTapioca::ContactBase::Presence,
                                       const QString &)),
                SLOT(onPresenceUpdated(QtTapioca::ContactBase*,
                                       QtTapioca::ContactBase::Presence,
                                       const QString &)));

        Q_ASSERT(0 != connection);
        Q_ASSERT(connection->status() == QtTapioca::Connection::Connected);
        emit connectionOpened(connection);
    }

    // Remove connection from list of known connections if it disconnected:
    if (new_state == QtTapioca::Connection::Disconnected)
    {
        Q_ASSERT(0 != connection);
        Q_ASSERT(connection->status() == QtTapioca::Connection::Disconnected);
        emit connectionClosed(connection);

        connection->deleteLater();
        int num_removals = d->connections.removeAll(connection);
        Q_ASSERT(num_removals == 1);
    }
}

void ConnectionFacade::onPresenceUpdated(QtTapioca::ContactBase * contact_info,
                                         QtTapioca::ContactBase::Presence presence_state,
                                         const QString & message)
{
    Q_ASSERT(contact_info != 0);
    const QtTapioca::UserContact* user_contact = dynamic_cast<const QtTapioca::UserContact*>
            (contact_info);
    if (user_contact == 0) { return; } // Not a user contact after all...

    // find the connection handle for this user_contact:
    QtTapioca::Connection * connection;
    foreach(connection, d->connections)
    {
        if (connection->userContact() == user_contact)
        {
            Q_ASSERT(connection != 0);
            Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
                     presence_state <= QtTapioca::ContactBase::Busy);
            emit ownPresenceUpdated(connection, int(presence_state), message);
            return;
        }
    }
}

void ConnectionFacade::onChannelOpened(QtTapioca::Connection * connection,
                                       QtTapioca::Channel * channel,
                                       bool suppress_handler)
{
    if (connection == 0 || channel == 0 || suppress_handler) { return; }

    // We must do something with this channel...
    Q_ASSERT(connection != 0);
    Q_ASSERT(channel != 0);
    Q_ASSERT(connection->status() == QtTapioca::Connection::Connected);
    emit channelOpened(connection, channel);
}

int ConnectionFacade::updatePresence(const QtTapioca::Connection * connection,
                                     const int presence_state,
                                     const QString & message)
{
    Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
             presence_state <= QtTapioca::ContactBase::Busy);

    if (0 == connection) { return Decibel::ERROR_NO_SUCH_CONNECTION; }

    Q_ASSERT(connection != 0);

    if (connection->userContact() == 0 ||
        !connection->userContact()->setPresenceMessage(message) ||
        !connection->userContact()->setPresence(QtTapioca::UserContact::Presence(presence_state))) // not connected!
    { return Decibel::ERROR_FAILED_TO_UPDATE_PRESENCE_INFORMATION; }

    return presence_state;
}

==> decibel_0.4.0.orig.tar.gz: decibel-0.4.0/src/server/connectionfacade.cpp <==
/*
 * the Decibel Realtime Communication Framework
 * Copyright (C) 2006 by basyskom GmbH
 *  @author Tobias Hunger <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <QtCore/QCoreApplication>
#include <QtCore/QPointer>

#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusConnectionInterface>

#include <QtTapioca/ConnectionManager>
#include <QtTapioca/ConnectionManagerFactory>
#include <QtTapioca/UserContact>

#include "connectionfacade.h"
#include "protocolmanager.h"

#include <Decibel/Errors>
#include <Decibel/AccountData>

/// @cond INCLUDE_PRIVATE

/**
 * @brief Private data class for the ConnectionFacade.
 * @author Tobias Hunger <info@basyskom.de>
 */
class ConnectionFacadePrivate
{
public:
    ConnectionFacadePrivate(ProtocolManager * const protocol_mgr) :
        protocolMgr(protocol_mgr),
        acceptConnections(true)
    { Q_ASSERT(0 != protocol_mgr); }

    /** @brief A pointer to the ProtocolManager. */
    ProtocolManager * const protocolMgr;

    /** @brief A list of known Connections. */
    QList<QtTapioca::Connection *> connections;

    bool acceptConnections;
};

/// @endcond

// -----------------------------------------------------------------------

ConnectionFacade::ConnectionFacade(ProtocolManager* protocol_mgr,
                                   QObject * parent) :
    QObject(parent),
    d(new ConnectionFacadePrivate(protocol_mgr))
{
    connect(QDBusConnection::sessionBus().interface(),
            SIGNAL(serviceOwnerChanged(const QString &, const QString &, const QString &)),
            this,
            SLOT(onServiceOwnerChanged(const QString &, const QString &, const QString &)));
}

ConnectionFacade::~ConnectionFacade()
{ delete d; }

void ConnectionFacade::onAboutToQuit()
{
    d->acceptConnections = false;

    emit shuttingDown();

    // Disconnect from all Connections:
    QtTapioca::Connection * connection;
    foreach (connection, d->connections)
    { connection->disconnect(); }

    // Wait for disconnection events:
    QCoreApplication * const qapp(QCoreApplication::instance());
    while (!d->connections.isEmpty())
    { qapp->processEvents(); }
}

QtTapioca::Connection *
ConnectionFacade::connectUsingAccountAndConnectionManager(const QVariantMap& account_data,
                                                          const QString cm_name,
                                                          const int presence_state,
                                                          const QString message)
{
    Q_ASSERT(!account_data.isEmpty());
    Q_ASSERT(account_data.contains(Decibel::name_protocol));

    if (!d->acceptConnections)
    { return 0; }

    QString protocol(account_data[Decibel::name_protocol].toString());

    Q_ASSERT(presence_state != QtTapioca::ContactBase::Offline);
    // AccountManager should not have called us if the account is
    // supposed to be offline!
    Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
             presence_state <= QtTapioca::ContactBase::Busy);

    // Get CM:
    QtTapioca::ConnectionManager * current_cm =
            QtTapioca::ConnectionManagerFactory::self()->
            getConnectionManagerByName(cm_name);

    if (current_cm == 0) { return 0; }

    Q_ASSERT(current_cm != 0);

    // Map parameters:
    QList<QtTapioca::ConnectionManager::Parameter> cm_params =
            current_cm->protocolParameters(protocol);
    QList<QtTapioca::ConnectionManager::Parameter> account_params;
    QtTapioca::ConnectionManager::Parameter current_param;
    QString key;
    foreach (current_param, cm_params)
    {
        key = current_param.name();
        if (account_data.contains(key) &&
            account_data[key].typeName() ==
                current_param.value().typeName())
        {
            account_params.append(QtTapioca::ConnectionManager::Parameter(key,
                                                                          account_data[key]));
        }
        else if (current_param.flags() & QtTapioca::ConnectionManager::Parameter::Required)
        { return 0; }
    }

    // Create connection:
    QtTapioca::Connection* connection = current_cm->requestConnection(protocol, account_params);
    if (connection == 0)
    { return 0; }

    Q_ASSERT(connection != 0);

    Q_ASSERT(!d->connections.contains(connection));
    d->connections.append(connection);

    // register connection:
    connect(connection,
            SIGNAL(statusChanged(QtTapioca::Connection*,
                                 QtTapioca::Connection::Status,
                                 QtTapioca::Connection::Reason)),
            this,
            SLOT(onStatusChanged(QtTapioca::Connection *,
                                 QtTapioca::Connection::Status,
                                 QtTapioca::Connection::Reason)));
    connect(connection,
            SIGNAL(channelCreated(QtTapioca::Connection*,
                                  QtTapioca::Channel*, bool)),
            this,
            SLOT(onChannelOpened(QtTapioca::Connection*,
                                 QtTapioca::Channel*, bool)));

    // do the actual connect:
    connection->connect(QtTapioca::UserContact::Presence(presence_state), message);

    return connection;
}

QtTapioca::Connection *
ConnectionFacade::connectUsingAccount(const QVariantMap& account_data,
                                      const int presence_state,
                                      const QString message)
{
    QString protocol = account_data[Decibel::name_protocol].toString();
    // Get a connection manager to use:
    QString cm_name(d->protocolMgr->defaultConnectionManagerForProtocol(protocol));
    if (cm_name.isEmpty())
    { return 0; }

    return connectUsingAccountAndConnectionManager(account_data, cm_name, presence_state, message);
}

void ConnectionFacade::onStatusChanged(QtTapioca::Connection* connection,
                                       QtTapioca::Connection::Status new_state,
                                       QtTapioca::Connection::Reason /* reason */)
{
    // find connection_handle:
    if (0 == connection) { return; }

    // This is one of our connections:
    Q_ASSERT(0 != connection); // 0 connection may not end up in d->connections!

    if (new_state == QtTapioca::Connection::Connected)
    {
        Q_ASSERT(0 != connection->userContact());
        addConnection(connection);
    }
    else if (new_state == QtTapioca::Connection::Disconnected)
    {
        Q_ASSERT(connection->status() == QtTapioca::Connection::Disconnected);
        rmConnection(connection);
    }
}

void ConnectionFacade::onPresenceUpdated(QtTapioca::ContactBase * contact_info,
                                         QtTapioca::ContactBase::Presence presence_state,
                                         const QString & message)
{
    Q_ASSERT(contact_info != 0);
    const QtTapioca::UserContact* user_contact =
            dynamic_cast<const QtTapioca::UserContact*>(contact_info);
    if (user_contact == 0) { return; } // Not a user contact after all...

    // find the connection handle for this user_contact:
    QtTapioca::Connection * connection;
    foreach(connection, d->connections)
    {
        if (connection->userContact() == user_contact)
        {
            Q_ASSERT(connection != 0);
            Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
                     presence_state <= QtTapioca::ContactBase::Busy);
            emit ownPresenceUpdated(connection, int(presence_state), message);
            return;
        }
    }
}

void ConnectionFacade::onChannelOpened(QtTapioca::Connection * connection,
                                       QtTapioca::Channel * channel,
                                       bool suppress_handler)
{
    if (connection == 0 || channel == 0 || suppress_handler)
    { return; }

    // We must do something with this channel...
    Q_ASSERT(connection != 0);
    Q_ASSERT(channel != 0);
    Q_ASSERT(connection->status() == QtTapioca::Connection::Connected);
    emit channelOpened(connection, channel);
}

int ConnectionFacade::updatePresence(const QtTapioca::Connection * connection,
                                     const int presence_state,
                                     const QString & message)
{
    Q_ASSERT(presence_state >= QtTapioca::ContactBase::Offline &&
             presence_state <= QtTapioca::ContactBase::Busy);

    if (0 == connection) { return Decibel::ERROR_NO_SUCH_CONNECTION; }

    Q_ASSERT(connection != 0);

    if (connection->userContact() == 0 ||
        !connection->userContact()->setPresenceMessage(message) ||
        !connection->userContact()->setPresence(QtTapioca::UserContact::Presence(presence_state))) // not connected!
    { return Decibel::ERROR_FAILED_TO_UPDATE_PRESENCE_INFORMATION; }

    return presence_state;
}

void ConnectionFacade::onServiceOwnerChanged(const QString & service,
                                             const QString & /* old_owner */,
                                             const QString & new_owner)
{
    if (!new_owner.isEmpty()) { return; }

    QtTapioca::Connection * connection;
    foreach (connection, d->connections)
    {
        if (connection->serviceName() == service)
        { rmConnection(connection); }
    }
}

void ConnectionFacade::addConnection(QtTapioca::Connection * const connection)
{
    Q_ASSERT(0 != connection);
    Q_ASSERT(0 != connection->userContact());
    Q_ASSERT(connection->status() == QtTapioca::Connection::Connected);

    connect(connection->userContact(),
            SIGNAL(presenceUpdated(QtTapioca::ContactBase*,
                                   QtTapioca::ContactBase::Presence,
                                   const QString &)),
            SLOT(onPresenceUpdated(QtTapioca::ContactBase*,
                                   QtTapioca::ContactBase::Presence,
                                   const QString &)));

    emit connectionOpened(connection);

    // Fake presence information for the AccountManager:
    if (!connection->hasPresenceSupport())
    {
        emit ownPresenceUpdated(connection,
                                int(QtTapioca::ContactBase::Available),
                                QString(""));
    }

}

void ConnectionFacade::rmConnection(QtTapioca::Connection * const connection)
{
    Q_ASSERT(0 != connection);

    // Fake presence information for the AccountManager:
    if (!connection->hasPresenceSupport())
    {
        emit ownPresenceUpdated(connection,
                                int(QtTapioca::ContactBase::Offline),
                                QString(""));
    }

    emit connectionClosed(connection);

    int num_removals = d->connections.removeAll(connection);
    Q_ASSERT(num_removals == 1);

    delete connection;
}
